#========================================================================================
# package      DE_EPAGES::Core::API::Program::Base
# state        public
#----------------------------------------------------------------------------------------
# description  Common OS-independend interface for 3rd party programs.
#               Sub classes must implement the abstract methods <function command> and
#               <function version>.
#----------------------------------------------------------------------------------------
# example      use MyProgram; # sub class of DE_EPAGES::Core::API::Program::Base
#               $Program = MyProgram->new;
#               say "MyProgram is installed" if $Program->isInstalled;
#               say "Version=".$Program->version;
#               system $MyProgram->command, '-v';
#========================================================================================
package DE_EPAGES::Core::API::Program::Base;

use strict;
use LWP::Simple;
use HTTP::Status;
use Scalar::Util qw( blessed );
use Sort::Versions qw(versioncmp);
use Time::HiRes qw (time);

use DE_EPAGES::Core::API::Error qw( CallerError ExistsError GetError );
use DE_EPAGES::Core::API::Program::Tools qw(Has64BitOS ProgramFileDir);
use DE_EPAGES::Core::API::SystemCommand qw( SystemCommand );

#ERRORDEFINITION
sub ErrorDefinition () {
    return {
        'NotImplemented' => 'Method "#Method" is not implemented for "#Package"',
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Program = SubClass->new;
# example      $Node = DE_EPAGES::Core::API::Program::nodejs->new;
#----------------------------------------------------------------------------------------
# description  Creates a new program object.
#----------------------------------------------------------------------------------------
# return       $Program | program object | DE_EPAGES::Core::API::Program::Base
#========================================================================================
sub new {
    my $class = shift;
    return bless {}, $class;
}

#========================================================================================
# function     command
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Command = $Program->command;
# example      SystemCommand( [ $Program->command, '-e', 'print "1+2"' ] );
# example      $Output = SystemCommand( '"'.$Program->command.'" -v', { Capture => 1 } );
#----------------------------------------------------------------------------------------
# description  Returns the executable command.
#               The result may or may not contain the full path name and extension.
#               Note: the command may contain spaces - therefore use quotes where required.
#               The method does not verify if the executable actually exists, has execute
#               permissions or is included in the search path.
#               Examples: <ul>
#                 <li>perl</li>
#                 <li>perl.exe</li>
#                 <li>/usr/local/bin/perl</li>
#                 <li>C:\Program Files\Perl-5.16.0\bin\perl.exe</li>
#               </ul>
#----------------------------------------------------------------------------------------
# return       $Command | executable command | string
#========================================================================================
sub command {
    my $self = shift;

    CallerError( 'NotImplemented', {
        Package => blessed($self),
        Method => 'command',
    });
}

#========================================================================================
# function     version
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Version = $Program->version;
# example      say "Version=".$Program->version;
#----------------------------------------------------------------------------------------
# description  Returns the current version number executable command, typically in the
#               format "0.0.0".
#               Throws an error if the program is not installed or is not executable by
#               the current user.
#               Possible error codes are: CommandError|CommandFailed|NoVersionFound.
#----------------------------------------------------------------------------------------
# return       $Version | version number | string
#========================================================================================
sub version {
    my $self = shift;

    CallerError( 'NotImplemented', {
        Package => blessed($self),
        Method => 'version',
    });
}

#========================================================================================
# function     winpkg
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package = $Program->winpkg;
#----------------------------------------------------------------------------------------
# description  Returns the current package name including suffix (e.g. .zip or .msi).
#----------------------------------------------------------------------------------------
# return       $Package | package name | string
#========================================================================================
sub winpkg {
    my $self = shift;

    CallerError( 'NotImplemented', {
        Package => blessed($self),
        Method => 'winpkg',
    });
}

#========================================================================================
# function     resource
# state        public
#----------------------------------------------------------------------------------------
# syntax       ($DownloadUrl, $DownloadVersion) = @{$Program->resource};
#----------------------------------------------------------------------------------------
# description  Returns available URL and version of the product on the download site.
#               If envvar EPAGES_REPO is set, prefer that URL (space-separated URL or
#               absolute path).
#----------------------------------------------------------------------------------------
# return       $DownloadUrl | download URL | string
# return       $DownloadVersion | download version | string
#========================================================================================
sub resource {
    my $self = shift;
    my $Program = (split /::/, blessed($self))[-1];
    my $MaxVersion = 0;
    my %Speed;

    my @UrlList = qw(http://ftp.epages.com/pub/epages/build/repo/epages
                     http://epages-software.de/repo/epages);
    unshift @UrlList, split(/\s+/,$ENV{EPAGES_REPO}) if $ENV{EPAGES_REPO};

    # first internal resources because they might contain newer versions
    foreach my $Url (@UrlList) {
        # stop if EPAGES_REPO found
        last if $ENV{EPAGES_REPO} and scalar keys %Speed;
        # stop if internal resource found
        last if $Url =~ /epages-software.de/ and scalar keys %Speed;

        my $time1 = time;
        my $Content = get("$Url/windows/3rd-party/versions.txt");
        next unless $Content;

        my $Version = (split /=/, (grep {/$Program=/} split(/\s+/, $Content))[0])[1];
        next unless $Version;

        $MaxVersion = $Version if versioncmp($MaxVersion, $Version) < 0;
        $Speed{$Version}{$Url} = time - $time1;
    }

    my $Fastest = (sort { $Speed{$MaxVersion}{$a} <=> $Speed{$MaxVersion}{$b} }
                   keys %{$Speed{$MaxVersion}})[0];

    my $Arch = Has64BitOS() ? 64 : 32; ## no critic qw(ProhibitMagicNumbers)
    my $Package = $self->winpkg;
    return ["$Fastest/windows/3rd-party/$Arch/$Package", $MaxVersion];
}

#========================================================================================
# function     isInstalled
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsInstalled = $Program->isInstalled;
#----------------------------------------------------------------------------------------
# description  checks if $Program is installed
#----------------------------------------------------------------------------------------
# return       $IsInstalled | installed version if installed, else 0 | string
#========================================================================================
sub isInstalled {
    my $self = shift;

    my $Version;
    # if program does not work it must be reinstalled
    eval { $Version = $self->version; };
    warn GetError()->message if ExistsError();
    return $Version ? $Version : 0;
}

#========================================================================================
# function     isUpToDate
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Program->update unless $Program->isUpToDate;
#----------------------------------------------------------------------------------------
# description  checks if $Program is up-to-date (not installed -> not up-to-date)
#----------------------------------------------------------------------------------------
# return       $IsUpToDate | current version if up-to-date, else 0 | string
#========================================================================================
sub isUpToDate {
    my $self = shift;
    my $InstalledVersion = $self->isInstalled;
    return 0 unless $InstalledVersion;

    my (undef, $DownloadVersion) = @{$self->resource};
    my $Outdated = versioncmp($DownloadVersion, $InstalledVersion) > 0;
    return $Outdated ? 0 : $InstalledVersion;
}

#========================================================================================
# function     update
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Update = $Program->update;
#----------------------------------------------------------------------------------------
# description  updates/installs $Program unless up-to-date
#----------------------------------------------------------------------------------------
# return       $Update | true if was updated, else false | boolean
#========================================================================================
sub update {
    my $self = shift;

    return 0 if $self->isUpToDate;

    ## no critic qw(ProhibitMagicNumbers)
    my $Program = (split /::/, blessed($self))[-1];
    my $Bits = $Program =~ /jdk|mongo/ ? undef : 32 ;
    ## use critic

    my $TargetDir = ProgramFileDir($Bits);
    my $StorePkg = "$TargetDir\\". $self->winpkg;
    my ($DownloadUrl, undef) = @{$self->resource};

    print "getstore($DownloadUrl, $StorePkg)\n";
    my $Result = getstore($DownloadUrl, $StorePkg);
    die "\nERROR retrieving: $DownloadUrl\nReturn code == $Result\n"
        unless $Result == RC_OK;

    $self->preInstall;

    my $Install = sprintf($self->installCmd, $StorePkg, $TargetDir);
    SystemCommand($Install, { 'Print' => 1 } );

    $self->postInstall;

    return 1;
}

#========================================================================================
# function     preInstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Program->preInstall
#----------------------------------------------------------------------------------------
# description  runs code to be executed before installing/updating $Program
#========================================================================================
sub preInstall {
    my $self = shift;
}

#========================================================================================
# function     postInstall
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Program->postInstall
#----------------------------------------------------------------------------------------
# description  runs code to be executed before installing/updating $Program
#========================================================================================
sub postInstall {
    my $self = shift;
}

#========================================================================================
# function     installCmd
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Cmd = $Program->installCmd;
#----------------------------------------------------------------------------------------
# description  Returns the install command for program
#----------------------------------------------------------------------------------------
# return       $Cmd | install command for program | string
#========================================================================================
sub installCmd {
    my $self = shift;

    CallerError( 'NotImplemented', {
        Package => blessed($self),
        Method => 'installCmd',
    });
}

1;
