#========================================================================================
# package      DE_EPAGES::Mollie::API::MolliePayment
# base         DE_EPAGES::EPagesJ::API::Object::RestClient
# state        public
# description  Client for Mollie
#========================================================================================
package DE_EPAGES::Mollie::API::MolliePayment;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject TestString);
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Mollie::API::PaymentRequest;
use DE_EPAGES::Mollie::API::Token;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );

use constant SF_TYPE         => 'sf';
use constant MOBILE_TYPE     => 'mobile';

sub ErrorDefinition {
    return {
        'PaymentRequestError'  => 'Error Creating the payment request at epagesj',
        'EmptyDataError'  => 'PaymentURL or TransactionID are empty',
        'MollieSiteError'  => 'Some error occurs when call Mollie REST'
    };
}
#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Mollie::API::MolliePayment->new( %args );
#----------------------------------------------------------------------------------------
# description  Creates a new MolliePayment
#----------------------------------------------------------------------------------------
# return       OAuthTokenClient | new instance | object
#========================================================================================
sub new {
    my $class = shift;

    my $self = bless { 'PaymentURL' => undef,
                       'TransactionID' => undef
                     }, $class;

    return $self;
}
#========================================================================================
# function     createPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentRequest = DE_EPAGES::Mollie::API::MolliePayment->createPayment($LineItemPayment, $Locale);
#----------------------------------------------------------------------------------------
# description  Creates a Payment Transaction at Mollie
#----------------------------------------------------------------------------------------
# input        $LineItemPayment | LineItemPayment Object | Object
# input        $Locale | $Locale Object | Object
# input        $isMobile | define if the customer is using sf or mobile view  | String
#========================================================================================
sub createPayment {
    my $self    = shift;
    my ($LineItemPayment, $Locale, $isMobile) = @_;

    TestObject('LineItemPayment',$LineItemPayment,'DE_EPAGES::Mollie::API::Object::LineItemPaymentMollie');
    TestString('Locale',$Locale);

    my $Shop = $LineItemPayment->get('Site');

    my $hPaymentRequest = $self->_createPaymentHash($LineItemPayment, $Locale, $isMobile);
    my $PaymentRequest  = $self->_createPaymentRequest($Shop, $Locale);
    my $apikey = $self->_getTokenNumber($Shop);

    my $hPaymentResponse;

    eval {$hPaymentResponse = $PaymentRequest->createRequest($hPaymentRequest,$apikey);};
    if ( ExistsError() ) {
        my $Error = GetError();
        $self->_LogPayment('Mollie','createPayment' , $Error->longMessage());
        Error('PaymentRequestError');
    }

    $self->_LogPayment('Mollie','createPayment' , $hPaymentResponse);

    if (isValid($hPaymentResponse->{'paymentUrl'}) && isValid($hPaymentResponse->{'id'})) {
        $self->{'PaymentURL'} = $hPaymentResponse->{'paymentUrl'};
        $self->{'TransactionID'} = $hPaymentResponse->{'id'};
    }
    elsif (isValid($hPaymentResponse->{'error'})){
        Error('MollieSiteError');
    }
    else {
        $self->_LogPayment('Mollie','createPayment' , 'PaymentURL or TransactionID are empty');
        Error('EmptyDataError');
    }

    return;
}
#========================================================================================
# function     listPayments
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentRequest = DE_EPAGES::Mollie::API::MolliePayment->listPayments($LineItemPayment, $Locale);
#----------------------------------------------------------------------------------------
# description  Creates a list of Mollie Payment Transaction 
#----------------------------------------------------------------------------------------
# input        $Shop | Shop Object | Object
# input        $Locale | $Locale Object | Object
# input        $TokenNumber | String | String
#========================================================================================
sub listPayments {
    my $self    = shift;
    my ($Shop, $Locale, $TokenNumber) = @_;

    TestString('Locale',$Locale);

    if (!isValid($TokenNumber)){
        $TokenNumber = $self->_getTokenNumber($Shop);
    }

    my $PaymentRequest  = $self->_createPaymentRequest($Shop, $Locale);
    my $hPaymentResponse;

    eval {$hPaymentResponse = $PaymentRequest->listPayments($TokenNumber)};
    if ( ExistsError() ) {
        my $Error = GetError();
        $self->_LogPayment('Mollie','listPayments' , $Error->longMessage());
        Error('PaymentRequestError');
    }

    $self->_LogPayment('Mollie','listPayments' , $hPaymentResponse);

    if (isValid($hPaymentResponse->{'totalCount'})) {
        if($hPaymentResponse->{'totalCount'} > 0){
            $self->{'data'} = $hPaymentResponse->{'data'};
        }
    }
    elsif (isValid($hPaymentResponse->{'error'})){
        Error('MollieSiteError');
    }
    else {
        $self->_LogPayment('Mollie','listPayments error' , 'Failed to get list of payments');
        Error('EmptyDataError');
    }

    return;
}
#========================================================================================
sub _createPaymentRequest {
    my $self = shift;
    my ($Shop, $Locale) = @_;

    return DE_EPAGES::Mollie::API::PaymentRequest->create(
        {
            Shop   => $Shop,
            Locale => $Locale
        });
}

#========================================================================================
sub _getTokenNumber {
    my $self = shift;
    my ($Shop) = @_;

    my $TokenObject = DE_EPAGES::Mollie::API::Token->new();
    return $TokenObject->getKey($Shop);
}

#==============================================================================
sub _getReturnURL {
    my $self = shift;
    my ($Shop, $GUID, $isMobile) = @_;

    my $Type = SF_TYPE;
    if ($isMobile) {
        $Type = MOBILE_TYPE;
    }

    return BuildShopUrl($Shop,
            {
                'ChangeAction' => 'PaymentBacklinkMollie',
                'MGUID'        => $GUID
            },
            {   'Type'   => $Type,
                'UseSSL' => 1,
                'UseObjectPath' => 1
            }
        );
}

#==============================================================================
sub _createPaymentHash {
    my $self = shift;
    my ($LineItemPayment, $Locale, $isMobile) = @_;

    my $hPaymentRequest;
    my $hMetaData;

    my $hLineItemPaymentSettings = $LineItemPayment->get(['Amount', 'GUID', 'Parent', 'PaymentMethod', 'Site']);

    my $Shop = $hLineItemPaymentSettings->{'Site'};
    my $hShopSettings = $Shop->get(['Alias', 'NameOrAlias', 'ProtocolAndServerSSL']);

    my $BasketOrder = $hLineItemPaymentSettings->{'Parent'}->parent();
    my $hBasketOrderSettings = $BasketOrder->get(['Alias', 'GUID']);

    $hPaymentRequest->{amount} = $hLineItemPaymentSettings->{'Amount'};
    $hMetaData->{guid} = $hBasketOrderSettings->{'GUID'};
    $hMetaData->{orderId} = $hBasketOrderSettings->{'Alias'};
    $hMetaData->{reseller} = $hLineItemPaymentSettings->{'PaymentMethod'}->get('Reseller');
    $hPaymentRequest->{metadata} = $hMetaData;
    $hPaymentRequest->{description} = $hShopSettings->{'NameOrAlias'};
    $hPaymentRequest->{redirectUrl} = $self->_getReturnURL($Shop, $hLineItemPaymentSettings->{'GUID'}, $isMobile);
    $hPaymentRequest->{webhookUrl} =
        $hShopSettings->{'ProtocolAndServerSSL'} . '/rs/shops/' . $hShopSettings->{'Alias'} . '/payments/mollie/update';
    $hPaymentRequest->{locale} = $Locale;

    return $hPaymentRequest;
}

#============================================================================
sub _LogPayment {
    my $self = shift;
    my ( $PaymentTypeAlias, $Name, $DebugStuff ) = @_;

    LogPayment( $PaymentTypeAlias, $Name, $DebugStuff );

    return;
}
1;
