#==============================================================================
# package      DE_EPAGES::PayPal::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::PayPal::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

use strict;
use DateTime::Duration;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError  );
use DE_EPAGES::Core::API::PerlTools qw ( fcmp );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByGUID
    LoadObjectByGUID
    LoadRootObject
);
use DE_EPAGES::Core::API::TestParameter qw (TestObject TestString);
use DE_EPAGES::Order::UI::Basket;
use DE_EPAGES::ThirdPartyConfig::API::ThirdPartyConfig qw ( GetDefaultTimeOut );

use constant MAX_TRANSACTION_AGE_MINUTES => 120; # minutes

#==============================================================================
sub SetTransStatus {
    my ($Order, $TransStatus) = @_;

    my $Now = GetCurrentDBHandle()->currentDateTime;
    $Order->set({'PendingOn',   $Now})  if $TransStatus eq 'pending';
    $Order->set({'PaidOn',      $Now})  if $TransStatus eq 'completed';
    $Order->set({'CancelledOn', $Now})  if $TransStatus eq 'denied';

    # EPG-27322: remove PaidOn status if payment was reversed
    $Order->set({'PaidOn',      undef}) if $TransStatus eq 'reversed';

    return;
}

#==============================================================================
sub PayPalIPN {
    my $self = shift;
    my ($Servlet) = @_;

    my $hFormValues = $Servlet->params;
    LogPayment('PayPal', 'PayPalIPN', $hFormValues);

    # Test the parameters of PayPal, show errors if something is wrong
    my $hValues = TestPayPalParametersIPN($Servlet, $hFormValues);
    return undef unless defined $hValues;

    # update the paymentlineitem
    my $PaymentLineItem = delete($hValues->{'PaymentLineItem'});

    # EPG-27322: reset IPN processed flag before verifing if IPN is sent because of an reversed payment
    $PaymentLineItem->set({'IsIPNProcessed' => 0}) if (defined $hValues->{'TransStatus'} and lc $hValues->{'TransStatus'} eq 'reversed');

    # EPG-26477: verify IPN message
    return undef unless _verifyPayPalIPN($Servlet->request, $PaymentLineItem);

    # EPG-27546: prevent creating an order if transaction status isn't positive and order creation occurs only if payment is successful
    my $DoPaymentOnBasket = $PaymentLineItem->get('PaymentMethod')->get('DoPaymentOnBasket');
    return undef if ($DoPaymentOnBasket and defined $hValues->{'TransStatus'} and not lc $hValues->{'TransStatus'} ~~ [qw(completed pending created processed)]);

    $PaymentLineItem->set($hValues);

    my $Object = $PaymentLineItem->container->parent;
    my $Order;
    if ($Object->instanceOf('Basket')) {
        eval {
            $Order = DE_EPAGES::Order::UI::Basket->doBasket2Order($Servlet, $Object);
        };
        if (ExistsError()) {
            my $Error = GetError();
            LogPayment('PayPal', 'PayPalIPN_Basket2Order_Error', $Error);
            Error($Error);
        }
    }
    else {
        $Order = $Object;
    }
    $Servlet->vars('ViewAction', 'View');
    $Servlet->vars('ViewObjectID', $Order->id);
    $Servlet->registerSession($Order->get('User'), 0);

    # reset cached language id etc.
    $Servlet->locale(1);
    $Servlet->languageID(1);
    $Servlet->currencyID(1);

    # set order status
    SetTransStatus($Order, lc($hValues->{'TransStatus'})) if $hValues->{'TransStatus'};

    # set IPN processed flag
    $PaymentLineItem->set({'IsIPNProcessed' => 1});

    return;
}

#==============================================================================
sub _verifyPayPalIPN {
    my ($Request, $PaymentLineItem) = @_;

    my $PaymentMethod = $PaymentLineItem->get('PaymentMethod');
    my $PaymentMethodAlias = $PaymentMethod->alias;

    # cancel if transaction was processed before
    if ($PaymentLineItem->get('IsIPNProcessed')) {
        LogPayment('PayPal', "Verifing IPN: reject IPN message for payment method $PaymentMethodAlias because it's already processed", {});
        return 0;
    }

    # resend IPN message
    # no need here to differ between Standard and Integral Evolution
    my $URL = LoadRootObject()->get($PaymentMethod->get('IsInTestMode') ? 'PayPal_ApiUrlSandbox' : 'PayPal_ApiUrl');

    # tell PayPal we want to verify that IPN message
    my $Query = $Request->content . '&cmd=_notify-validate';

    LogPayment('PayPal', "Verifing IPN: query to send to $URL for payment method $PaymentMethodAlias", $Query);

    my $VerifyRequest = HTTP::Request->new('POST', $URL);
    my $UserAgent = LWP::UserAgent->new();
    $UserAgent->timeout(GetDefaultTimeOut);
    $UserAgent->agent('ePages/6 ' . $UserAgent->agent());
    $VerifyRequest->content_type('application/x-www-form-urlencoded');
    $VerifyRequest->header('Host' => 'www.paypal.com');
    $VerifyRequest->content($Query);

    my $Response = $UserAgent->request($VerifyRequest);

    # Check verification response
    if ($Response->is_success) {
        LogPayment('PayPal', "Verifing IPN: response for payment method $PaymentMethodAlias", $Response->content);
        return 1 if $Response->content eq 'VERIFIED';
    }
    else {
        # request failed
        LogPayment('PayPal', "Verifing IPN: response for payment method $PaymentMethodAlias", $Response);
    }

    # verification not successful
    return 0;
}


#==============================================================================
sub PayPalRedirect {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    LogPayment('PayPal', 'PayPalRedirect-Form', $Form);

    my %FormValues = ();
    foreach my $Attribute (qw (amt cc tx st PaymentGUID mc_gross mc_currency txn_id payer_status payment_status)) {
        $FormValues{$Attribute} = $Form->value($Attribute);
    }
    LogPayment('PayPal', 'PayPalRedirect', \%FormValues);

    # test the parameters of PayPal, show errors if something is wrong
    my $hValues = TestPayPalParametersRedirect($Servlet, \%FormValues);
    # get payment line item
    my $PaymentLineItem = delete($hValues->{'PaymentLineItem'});

    # EPG-24383: ignore already processed redirects
    return if defined $PaymentLineItem->get('Field2') and $PaymentLineItem->get('Field2') eq 'PayPalRedirect';

    # EPG-19916: Check age of payment transaction
    my $TransTime = $PaymentLineItem->get('TransTime') // $hValues->{'TransTime'};
    if (defined $TransTime) { # undef means the transaction did not yet have any callback
        $TransTime->add(DateTime::Duration->new('minutes' => MAX_TRANSACTION_AGE_MINUTES));
        if ($TransTime < GetCurrentDBHandle()->currentDateTime) {
            $Servlet->handleObjectNotFound(DE_EPAGES::Core::API::Error->new(
                'Code' => 'PAYMENT_LINEITEM_TOO_OLD',
                'Vars' => { 'URI'=> $Servlet->request->uri }
            ));
            return;
        }
    }

    # update the payment line item
    $PaymentLineItem->set($hValues);

    my $Object = $PaymentLineItem->container->parent;
    my $Order = $Object->instanceOf('Basket')  ?  DE_EPAGES::Order::UI::Basket->doBasket2Order($Servlet, $Object) : $Object;

    # redirect to order confirmation page
    $Servlet->vars('ViewAction', 'View');
    $Servlet->vars('ViewObjectID', $Order->id);
    $Servlet->registerSession($Order->get('User'), 0);

    # reset cached language id etc.
    $Servlet->locale(1);
    $Servlet->languageID(1);
    $Servlet->currencyID(1);

    # set order status
    SetTransStatus($Order, lc($hValues->{'TransStatus'})) if $hValues->{'TransStatus'};

    # EPG-24383: mark PayPalRedirect as processed
    $PaymentLineItem->set({'Field2' => 'PayPalRedirect'});

    return;
}
#----------------------------------------------------------------------------------------
sub _TestRefererUrl {
    my ($Servlet, $PaymentLineItem) = @_;

    my $HttpReferer = $Servlet->params('HTTP_REFERER');
    TestString('(Is SSL activated?) HttpReferer', $HttpReferer);

    if ( $PaymentLineItem->get('IsTestTransaction') ) {
        if( $HttpReferer !~ m!https.*paypal\.com.*! ){
            Error( 'HTTP_REFERER is not mapping. (Testmode)', { 'actualUrl' => $HttpReferer, 'expected' => 'https.*paypal\.com' } );
        }
    } else {
        if( $HttpReferer !~ m!https.*paypal\.com.*! || $HttpReferer =~ m!https.*sandbox.paypal.com.*! ){
            Error( 'HTTP_REFERER is not mapping.', { 'actualUrl' => $HttpReferer, 'expected' => 'https.*paypal.com or not https.*sandbox.paypal.com' } );
        }
    }

    return;
}
#==============================================================================
sub TestPayPalParametersRedirect {
    my ($Servlet, $hFormValues) = @_;

    my $Shop = $Servlet->object;
    my $Form = $Servlet->form;
    my ($Check, $Amount, $Currency, $TransactionID, $TransactionStatus);
    eval {
        my $PaymentLineItem = _LoadLineItemPaymentPayPalByGuid($hFormValues->{'PaymentGUID'});
        _TestRefererUrl($Servlet, $PaymentLineItem);
        ($Check, $Amount, $Currency, $TransactionID, $TransactionStatus) = _doValidation($Shop, $hFormValues);
    };
    if (ExistsError()) {
        my $Error = GetError();
        $Error->addVars({'ShopAlias' => $Shop->alias()});
        LogError( $Error );
        $Form->mapFormError({
            'Name'       => 'amount',
            'ViewAction' => 'PayPalRedirectError'
        }, $Error);
        $Form->executeFormError();
    }

    return _prepareValues($hFormValues, $Check, $Amount, $Currency, $TransactionID, $TransactionStatus);
}

#==============================================================================
sub TestPayPalParametersIPN {
    my ($Servlet, $hFormValues) = @_;
    my $Shop = $Servlet->object;

    my ($Check, $Amount, $Currency, $TransactionID, $TransactionStatus);
    eval {
        ($Check, $Amount, $Currency, $TransactionID, $TransactionStatus) = _doValidation($Shop, $hFormValues);
    };
    if (ExistsError()) {
        Warning(GetError());
        return undef;
    }

    return _prepareValues($hFormValues, $Check, $Amount, $Currency, $TransactionID, $TransactionStatus);
}

#==============================================================================
sub _prepareValues {
    my ($hFormValues, $Check, $Amount, $Currency, $TransactionID, $TransactionStatus) = @_;

    my $PaymentLineItem = LoadObjectByGUID($hFormValues->{'PaymentGUID'} );
    my $Now = GetCurrentDBHandle()->currentDateTime;

    my %Values = (
        'PaymentLineItem' => $PaymentLineItem,
        'TransTimeMod'    => $Now,
        'IsSecure'        => defined($hFormValues->{'payer_status'}) && lc($hFormValues->{'payer_status'}) eq 'verified' ? 1 : 0,
    );
    $Values{'Amount'}        = $Amount                          if defined $Amount;
    $Values{'CurrencyID'}    = $Currency                        if defined $Currency;
    $Values{'PayerID'}       = $hFormValues->{'payer_id'}       if defined $hFormValues->{'payer_id'};
    $Values{'PendingReason'} = $hFormValues->{'pending_reason'} if defined $hFormValues->{'pending_reason'};


    if ($Check and defined $TransactionStatus) {
        $Values{'TransStatus'} = $TransactionStatus;
    }
    elsif (not $PaymentLineItem->get('TransStatus')) {
        $Values{'TransStatus'} = 'NotValidatedYet';
    }
    $Values{'TransID'}   = $TransactionID if defined $TransactionID;
    $Values{'TransTime'} = $Now unless defined $PaymentLineItem->get('TransTime');
    $Values{'Field1'}    = $hFormValues->{'payer_email'} if defined $hFormValues->{'payer_email'};

    return \%Values;
}

#==============================================================================
sub _LoadLineItemPaymentPayPalByGuid {
    my ($PaymentLineItemGUID) = @_;

    unless (defined($PaymentLineItemGUID) and ExistsObjectByGUID($PaymentLineItemGUID)) {
        Error('BadRedirect_NoPaymentGUID', {
            'Name'   => 'PaymentGUID',
            'Reason' => 'BadRedirect_NoPaymentGUID'
        });
    }

    my $PaymentLineItem = LoadObjectByGUID($PaymentLineItemGUID);
    TestObject('PaymentLineItem', $PaymentLineItem, 'DE_EPAGES::PayPal::API::Object::LineItemPaymentPayPal');

    return $PaymentLineItem;
}
#==============================================================================
sub _doValidation {
    my ($Shop, $hFormValues) = @_;
    my $Check = 1;
    my ($Amount, $Currency, $TransactionID, $TransactionStatus);

    my $PaymentLineItem = _LoadLineItemPaymentPayPalByGuid($hFormValues->{'PaymentGUID'});

    if ($PaymentLineItem->getSite()->id != $Shop->id) {
        Error('BadRedirect_ShopIDMisMatch', {
            'Name'   => 'PaymentGUID',
            'Reason' => 'BadRedirect_ShopIDMisMatch'
        });
    }

    my $LineItemContainer = $PaymentLineItem->get('LineItemContainer');
    my $GrandTotal = $LineItemContainer->get('GrandTotal');
    my $CurrencyID = $LineItemContainer->get('CurrencyID');


    # PayPal sends either amt/cc/tx/st or mc_gross/mc_currency/txn_id/payment_status

    if (defined $hFormValues->{'amt'}) {
        # check amt
        if (fcmp($GrandTotal, $hFormValues->{'amt'}) != 0) {
            LogPayment('PayPal', 'BadRedirect_AmountMisMatch GrandTotal:', $GrandTotal);
            LogPayment('PayPal', 'BadRedirect_AmountMisMatch amt:', $hFormValues->{'amt'});
        }

        # check cc
        if (defined $hFormValues->{'cc'} and $CurrencyID ne $hFormValues->{'cc'}) {
            LogPayment('PayPal', 'BadRedirect_AmountMisMatch CurrencyID:', $CurrencyID);
            LogPayment('PayPal', 'BadRedirect_AmountMisMatch cc:', $hFormValues->{'cc'});
        }

        # everything's ok
        if ($Check) {
            $Amount            = $hFormValues->{'amt'};
            $Currency          = $hFormValues->{'cc'};
            $TransactionID     = $hFormValues->{'tx'};
            $TransactionStatus = $hFormValues->{'st'};
        }
    }

    if (defined $hFormValues->{'mc_gross'}) {
        # check mc_gross
        if (fcmp($GrandTotal, $hFormValues->{'mc_gross'}) != 0) {
            LogPayment('PayPal', 'BadRedirect_AmountMisMatch GrandTotal:', $GrandTotal);
            LogPayment('PayPal', 'BadRedirect_AmountMisMatch mc_gross:', $hFormValues->{'mc_gross'});
        }

        # check mc_currency
        if (defined $hFormValues->{'mc_currency'} and $CurrencyID ne $hFormValues->{'mc_currency'}) {
            LogPayment('PayPal', 'BadRedirect_AmountMisMatch CurrencyID:', $CurrencyID);
            LogPayment('PayPal', 'BadRedirect_AmountMisMatch mc_currency:', $hFormValues->{'mc_currency'});
        }

        # everything's ok
        if ($Check) {
            $Amount            = $hFormValues->{'mc_gross'};
            $Currency          = $hFormValues->{'mc_currency'};
            $TransactionID     = $hFormValues->{'txn_id'};
            $TransactionStatus = $hFormValues->{'payment_status'};
        }
    }

    return ($Check, $Amount, $Currency, $TransactionID, $TransactionStatus);
}

#========================================================================================
# function     TransactionPaymentName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName();
#----------------------------------------------------------------------------------------
# description  return PayPal as string. Name of PaymentType for search/view
#               transactions.
#----------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName { 'PayPal' }

1;

