#==============================================================================
# package      DE_EPAGES::Product::API::WebService::ProductImageService
# base         DE_EPAGES::Shop::API::WebService::BaseShopService
# state        public
# description  Implements the Product image WebService.
#==============================================================================
package DE_EPAGES::Product::API::WebService::ProductImageService;
use base DE_EPAGES::Shop::API::WebService::BaseShopService;

use strict;
use Config;
use DE_EPAGES::Core::API::Log qw(LogDebug LogError);
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::Core::API::File qw (ConvertPathChars fsplit WriteFile ExistsFile DeleteFile);
use DE_EPAGES::Product::API::Product qw(ScaleImageSizes DeleteProductImage);
use DE_EPAGES::Product::API::Shop qw(GetProductImageSizes);

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Service = DE_EPAGES::Product::API::WebService::ProductImageService->new( %Options );
#----------------------------------------------------------------------------------------
# description  Creates a new web service object as a blessed hash reference and adds
#               the values from the %Options hash.
#----------------------------------------------------------------------------------------
# input        %Options | user-defined values | hash
# return       $Service | web service object | object
#========================================================================================
sub new {
    my $self = shift;
    my @Options = @_;
    $self->SUPER::new((
        'ServiceUrn' => 'urn://epages.de/WebService/ProductImageService/2006/03',
        'ServiceTypesUrn' => 'urn://epages.de/WebService/ProductImageTypes/2006/03',
        @Options,
    ));
}

#========================================================================================
# function     exists
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->exists( $ObjectPath, $FileName )
#----------------------------------------------------------------------------------------
# description  WebService-Method: exists
#               Returns exists status of a file in the given path.
#----------------------------------------------------------------------------------------
# input        $ObjectPath | object path | string
# input        $FileName | filename | string
# return       $exists  | 0/1 | boolean
#========================================================================================
sub exists {
    my $self = shift;

    # expects a list of product paths
    my ($ObjectPath, $FileName) = @_;
    LogDebug('ObjectPath=', $ObjectPath);
    LogDebug('FileName=', $FileName);

    $FileName = _ConvertPathChars($FileName);
    LogDebug('Converted FileName=', $FileName);

    my $Path = $self->loadShopObject( $ObjectPath, 'Product' )->get('PublicPath');
    my $File = $Path.'/'.$FileName;
    LogDebug('File=', $File);

    # build return object
    return $self->soapBool(ExistsFile($File))->name('Exists');
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->delete( $ObjectPath, $aFileNames )
#----------------------------------------------------------------------------------------
# description  WebService-Method: delete
#               Deletes a list of product attachments.
#----------------------------------------------------------------------------------------
# input        $ObjectPath | object paths | string
# input        $aFileNames | list of file names | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            delete status of all given files | SOAP::Data
#========================================================================================
sub delete {
    my $self = shift;

    # expects a list of product paths
    my ($ObjectPath, $aFileNames) = @_;
    LogDebug('ObjectPath=', $ObjectPath);
    LogDebug('aFileNames=', $aFileNames);

    my $Product = $self->loadShopObject( $ObjectPath, 'Product' );
    my $Path = $Product->get('PublicPath');

    my @DeletedFiles = ();
    foreach my $FileName (@$aFileNames) {

        $FileName = _ConvertPathChars($FileName);
        LogDebug('Converted FileName=', $FileName);

        my $File = $Path.'/'.$FileName;
        LogDebug('File=', $File);

        my $soapFile;
        eval {
            my $hOutFile = {
                'Path' => $self->soapString( $File ),
                'deleted' => $self->soapBool( DeleteFile($File) ),
            };

            DeleteProductImage( $Product, $FileName );

            my $SlideShowString = $Product->get('ImagesSlideShowString');
            $SlideShowString =~ s/$FileName\;//; #remove $FileName and \;
            $Product->set( { 'ImagesSlideShowString' => $SlideShowString } );

            # build return object
            $soapFile = $self->soapData('File', $hOutFile, 'TDelete_Return');
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapFile = $self->soapData('File', {
                'Path'       => $self->soapString( $File ),
                'Error'      => $self->soapError($Error)
            }, 'TDelete_Return');
        }
        push @DeletedFiles,$soapFile;
    }

    $self->mapType([qw (TDelete_Return)], $self->getServiceTypesUrn);
    return $self->soapData('Files', $self->soapList(\@DeletedFiles, 'TDelete_Return') );
}

#========================================================================================
# function     upload
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->upload( $ObjectPath,$hImageData)
#----------------------------------------------------------------------------------------
# description  WebService-Method: upload
#               Upload files to the public path of the given object by the objectpath.
#               The content of the Files are located in $hImageData
#----------------------------------------------------------------------------------------
# input        $ObjectPath | object path | string
# input        $hImageData | contains the content and the filename of the image  | ref.hash.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            status of the upload results | SOAP::Data
#========================================================================================
sub upload {
    my $self = shift;

    # expects a product path
    my ($ObjectPath,$hImageData) = @_;
    LogDebug('ObjectPath=', $ObjectPath);

    my $WriteOK;
    my $File;
    my @SavedFiles = ();

    # save attachments
    my $Object = $self->loadShopObject($ObjectPath, 'Product');
    my $Path = $Object->get('PublicPath');

    foreach my $part (@{$hImageData->{'ImageData'}}) {

        my $FileName = $part->{'FileName'};
        LogDebug('FileName=', $FileName);

        $FileName = _ConvertPathChars($FileName);
        LogDebug('Converted FileName=', $FileName);

        $File = $Path.'/'.$FileName;
        my $Content = $part->{'Content'};

        eval {
            WriteFile($File, \$Content, { binary=>1, create_directory=>1} );
        };
        if (ExistsError()) {
            my $Error = GetError();
            LogError($Error);

        } else { $WriteOK = 1; }

        # set attribute...
        if ($WriteOK) {
            my $AddAttr;
            $AddAttr->{'ImageLarge'} = $FileName;
            LogDebug('Attr',$AddAttr);
            $Object->set($AddAttr);
        }

        my $hOutFile = {
            'WriteOK'   => $self->soapBool( $WriteOK ),
            'File'     => $self->soapString( $File  ),
        };

        my $soapFile = $self->soapData('File', $hOutFile, 'TUpload_Return');
        push @SavedFiles,$soapFile;
    }

    # set the ImagesSlideShowString
    my $SlideShowString = $hImageData->{'ImagesSlideShowString'} ? $hImageData->{'ImagesSlideShowString'} : undef ;
    $Object->set({'ImagesSlideShowString' => $SlideShowString }) if defined $SlideShowString;

    if (not scalar @{$hImageData->{'ImageData'}}) {
        my $soapFile = $self->soapData('File', {
            'WriteOK'   => $self->soapBool( 0 ),
            'File'     => $self->soapString( 'no files'  )
        }, 'TUpload_Return');
        push @SavedFiles,$soapFile;
    }

    $self->mapType([qw (TUpload_Return)], $self->getServiceTypesUrn);
    return $self->soapData('Files', $self->soapList(\@SavedFiles, 'TUpload_Return') );
}

#========================================================================================
# function     uploadscale
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->uploadscale( $ObjectPath,$hImageData,$hParams)
#----------------------------------------------------------------------------------------
# description  WebService-Method: uploadscale
#               Upload a file to the public path of the given object by the objectpath.
#               Scale the attachments in $hImageData to the given sizes in $hParams if image
#----------------------------------------------------------------------------------------
# input        $ObjectPath | object path | string
# input        $hImageData | contains the content and the filename of the image  | ref.hash.array.string
# input        $hParams    | the scale vars |
# return       $SoapData  | a SOAP::Data structure with the
#                            upload and scale status of the file  | SOAP::Data
#========================================================================================
sub uploadscale {
    my $self = shift;

    # expects a list of product paths
    my ($ObjectPath, $hImageData, $hParams) = @_;

    LogDebug('ObjectPath=', $ObjectPath);
    LogDebug('hParams=', $hParams);

    my $WriteOK;
    my $File;
    my @SavedFiles;

    # save attachments
    my $Product = $self->loadShopObject( $ObjectPath, 'Product' );
    my $Path = $Product->get('PublicPath');
    my $hProductImageSizes = GetProductImageSizes($Product->getSite());

    foreach my $part (@{$hImageData->{'ImageData'}}) {

        my $FileName = $part->{'FileName'};
        LogDebug('FileName=', $FileName);

        $FileName = _ConvertPathChars($FileName);
        LogDebug('Converted FileName=', $FileName);

        $File = $Path.'/'.$FileName;
        my $Content = $part->{'Content'};

        eval {
            WriteFile($File, \$Content, { binary=>1, create_directory=>1} );
        };

        if (ExistsError()) {
            my $Error = GetError();
            LogError($Error);

        } else { $WriteOK = 1; }

        # convert parameters, because the syntax of ScaleImageSizes has changed;
        # parameters SCALE_BORDER_IMAGE_* and SCALE_BORDER_COLOR_* are ignored
        my %ScaleParams = map { $_ => $hParams->{$_} } grep { /SCALE_NAME_/ } keys %$hParams;

        foreach my $Key ( grep { /^SCALE_SIZE_/ } keys %$hParams ) {
            my $Suffix;
            if ( $Key =~ /^SCALE_SIZE_(.+)/ ) {
                $Suffix = $1;
            } else {
                die "invalid parameter key $Key";
            }
            $ScaleParams{'SCALE_WIDTH_'.$Suffix} = $hParams->{$Key};
            $ScaleParams{'SCALE_HEIGHT_'.$Suffix} = $hParams->{$Key};
        }

        # always create a thumbnail image
        $ScaleParams{'SCALE_WIDTH_xs'} = $hProductImageSizes->{'xs'}->[0];
        $ScaleParams{'SCALE_HEIGHT_xs'} = $hProductImageSizes->{'xs'}->[1];
        $ScaleParams{'SCALE_NAME_xs'} = 'ImageThumbnail';

        my $ScaleOK;
        if ($WriteOK) {
            # scale the image and add border
            $Product->set(ScaleImageSizes($File, \%ScaleParams));
            $ScaleOK = 1;
        }
        my $ProductWebPath = $Product->get('WebPath');
        my $FileWebPath = $ProductWebPath.'/'.$FileName;

        my $hOutFile = {
            'ScaleOK' => $self->soapBool( $ScaleOK ),
            'WriteOK' => $self->soapBool( $WriteOK ),
            'File' => $self->soapString( $FileWebPath  ),
        };

        my $soapFile = $self->soapData('File', $hOutFile, 'TUploadScale_Return');
        push @SavedFiles,$soapFile;
    }

    # set the ImagesSlideShowString
    my $SlideShowString = $hImageData->{'ImagesSlideShowString'} ? $hImageData->{'ImagesSlideShowString'} : undef ;
    $Product->set({'ImagesSlideShowString' => $SlideShowString }) if defined $SlideShowString;

    if (not scalar @{$hImageData->{'ImageData'}}) {
        my $soapFile = $self->soapData('File', {
            'WriteOK'   => $self->soapBool( 0 ),
            'ScaleOK'   => $self->soapBool( 0 ),
            'File'     => $self->soapString( 'no files'  )
        }, 'TUploadScale_Return');
        push @SavedFiles,$soapFile;
    }

    $self->mapType([qw (TUploadScale_Return)], $self->getServiceTypesUrn);
    return $self->soapData('Files', $self->soapList(\@SavedFiles, 'TUploadScale_Return') );
}

# convert FileNames like "Mlliardr.jpg"
sub _ConvertPathChars {
    my ($FileName) = @_;
    my ($Path, $FName, $Extension) = fsplit($FileName);
    my $ConvFileName = ConvertPathChars($FName);
    return "$Path"."$ConvFileName"."$Extension" ;
}

1;
