use strict;
use Getopt::Long;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError );
use DE_EPAGES::Core::API::File qw ( DeleteFile ExistsFile );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadRootObject ExistsObject );
use DE_EPAGES::SEO::API::XMLSitemap qw ( ExportXMLSitemap );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );

sub Main {
    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'              => \$Help,
        'passwd=s'          => \$Password,
        'storename=s'       => \$StoreName,
        'nooutput'          => \$NoOutput,
    );

    Usage() if $Help;
    unless( $StoreName ) {
        print STDERR "Missing parameter -storename\n\n";
        Usage();
    }

    my $time1 = time;
    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub {
            my $ShopContainer = LoadRootObject()->child('Shops');
            foreach my $ShopID (@{$ShopContainer->findChildren}) {
                next unless ExistsObject($ShopID);

                my $Shop = LoadObject($ShopID);
                next if ( $Shop->featureMaxValue('Unity') );

                my $hShopVars = $Shop->get([ qw(
                    ClosedByProvider
                    ClosedByMerchant
                    IsSitemapsOutdated
                    MobileSitemapPathFileName
                    ProtocolAndServer
                    SitemapPathFileName
                    SubmittedSitemapDomain
                ) ]);

                # reset SubmittedSitemapDomain if the shop domain changed
                if ($hShopVars->{'SubmittedSitemapDomain'} ne $hShopVars->{'ProtocolAndServer'}) {
                    $Shop->set({ 'SubmittedSitemapDomain' => undef });
                }
                # delete sitemaps if shop is closed
                if ($hShopVars->{'ClosedByProvider'} || $hShopVars->{'ClosedByMerchant'}) {
                    DeleteFile($hShopVars->{'SitemapPathFileName'})if ExistsFile($hShopVars->{'SitemapPathFileName'});
                    DeleteFile($hShopVars->{'MobileSitemapPathFileName'}) if ExistsFile($hShopVars->{'MobileSitemapPathFileName'});
                    next;
                }
                # continue unless the sitemap is outdated
                next unless $hShopVars->{'IsSitemapsOutdated'};
                # finally, update the sitemap
                ExportXMLSitemap($Shop);
                $Shop->set({ 'IsSitemapsOutdated' => 0 });
            }
        }
    );

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();

    my $time2 = time;
    printf "need %.1f seconds\n", $time2-$time1 if !$NoOutput;
}

sub Usage {
    print <<END_USAGE;
Usage:
    perl $0 [options] [flags]

description:
    Regenerates all sitemap xml files in all shops where the sitemaps
    are marked as outdated.

options:
    -passwd     database user password                  ()
    -storename  name of store                           (required)

flags:
    -help       show the command line options
    -nooutput   do not print any output

Example:
    perl $0 -storename Store
    perl $0 -storename Store -nooutput
END_USAGE
    exit 2;
}

RunScript( Sub=>\&Main );