#========================================================================================
# package      DE_EPAGES::Shop::API::Url
# state        public
# description  Some functions to build shop urls.
#========================================================================================
package DE_EPAGES::Shop::API::Url;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw ( BuildShopUrl
                      BuildDefaultShopUrl );

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::TestParameter qw ( TestHash TestObject );
use DE_EPAGES::Core::API::Url qw ( BuildParameterString UrlEncodeValue );
use DE_EPAGES::Presentation::Servlet::Permission;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'NO_SHOP_OBJECT' => 'Object #ObjectID (#ObjectPath) is not a shop object.'
    }
}
#ERRORDEFINITION END


#========================================================================================
# function     BuildShopUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $url = BuildShopUrl($Object, $hParams, $hOptions, $Servlet);
# example      $url = BuildShopUrl($PaymentMethod, {'ChangeAction' => 'Callback', 'ViewAction' => 'ViewResult'});
# example      $url = BuildShopUrl($Shop, {'ChangeAction' => 'SaveDate', 'ViewAction' => 'ViewResult'});
# example      $url = BuildShopUrl($PaymentMethod, {'ChangeAction' => ['BlockMonitor', 'Callback'], 'ViewAction' => 'ViewResult'});
#----------------------------------------------------------------------------------------
# description  Builds an epages url for a given shop object with the given get parameters
#----------------------------------------------------------------------------------------
# input        $Object | any object that belongs to a shop | object
# input        $hParams | url get parameters; values may be strings or arrays of strings | ref.hash
# input        $hOptions | with following keys:
#               <ul>
#                   <li>Type          - one of 'sf', 'admin', 'mobile', 'sf/mobile',
#                   if type is 'sf/mobile' a .mobile will be generated if the servlet is defined and the client
#                   is a mobile device, otherwise a .sf url is generated; defaults to 'sf' - string
#                   <li>UseSSL        - use https as protocol if SSL enabled for shop / database (optional, defaults to 0) - boolean
#                   <li>UseObjectPath - use object path rather than objectid (optional, defaults to 0) - boolean
#                   <li>AddToken      - add security token or not (optional, defaults to 0) - boolean
#                   <li>AddCookies    - add cookies to url or not (optional, defaults to 0) - boolean
#                   <li>NoUTF8        - do not use utf-8 for encoding get parameters (optional, defaults to 0) - boolean
#               </ul>  | ref.hash
# input        $Servlet | the current servlet (optional, but required if either AddToken or AddCookies is true)| object
# return       $url | the url | string
#========================================================================================
sub BuildShopUrl {
    my ($Object, $hParams, $hOptions, $Servlet) = @_;
    TestObject('Object', $Object, 'DE_EPAGES::Object::API::Object::Object');
    TestHash('hParams', $hParams, {});
    TestHash('hOptions', $hOptions, {
        'Type'          => 'String',
        'UseSSL'        => 'Integer',
        'UseObjectPath' => 'Integer',
        'AddToken'      => 'Integer',
        'AddCookies'    => 'Integer'
    });

    my $Shop = $Object->getSite();
    Error('NO_SHOP_OBJECT', {'ObjectID' => $Object->id, 'ObjectPath' => $Object->pathString}) unless $Shop->instanceOf('Shop');

    # add servlet params to options
    $hOptions->{'Cookies'} = ($hOptions->{'AddCookies'} && defined($Servlet)) ? $Servlet->cookies : undef;
    $hOptions->{'IsMobileServlet'} = ($hOptions->{'Type'} eq 'sf/mobile' && defined($Servlet)) ? $Servlet->vars('IsMobile') : undef;
    $hOptions->{'Token'} = ($hOptions->{'AddToken'} && defined($Servlet)) ? $Servlet->createSecurityToken : undef;

    return BuildDefaultShopUrl($Object, $hParams, $hOptions, $Shop) unless $Object->instanceOf('ShopObject');
    return $Object->getShopUrl($hParams, $hOptions, $Shop);
}

#========================================================================================
# function     BuildDefaultShopUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       $url = BuildDefaultShopUrl($Object, $hParams, $hOptions, $Shop);
#----------------------------------------------------------------------------------------
# description  Builds an epages url for a given shop object with the given get parameters
#----------------------------------------------------------------------------------------
# input        $Object | any object that belongs to a shop | object
# input        $hParams | url get parameters; values may be strings or arrays of strings | ref.hash
# input        $hOptions | with following keys:
#               <ul>
#                   <li>Type          - one of 'sf', 'admin', 'mobile', 'sf/mobile',
#                   if type is 'sf/mobile' a .mobile will be generated if the servlet is defined and the client
#                   is a mobile device, otherwise a .sf url is generated; defaults to 'sf' - string
#                   <li>UseSSL        - use https as protocol if SSL enabled for shop / database (optional, defaults to 0) - boolean
#                   <li>UseObjectPath - use object path rather than objectid (optional, defaults to 0) - boolean
#                   <li>AddToken      - add security token or not (optional, defaults to 0) - boolean
#                   <li>Token         - the security token (required AddToken is true) - String
#                   <li>AddCookies    - add cookies to url or not (optional, defaults to 0) - boolean
#                   <li>Cookies       - cookie values (required if AddCookies is true) - hash.ref
#                   <li>NoUTF8        - do not use utf-8 for encoding get parameters (optional, defaults to 0) - boolean
#                   <li>IsMobileServlet - is the client a mobile device or not - boolean
#               </ul>  | ref.hash
# input        $Shop | the current shop | object
# return       $url | the url | string
#========================================================================================
sub BuildDefaultShopUrl {
    my ($Object, $hParams, $hOptions, $Shop) = @_;

    # type defaults to sf
    my $Type = $hOptions->{'Type'} // 'sf';

    #--- add object id or object path
    if ($hOptions->{'UseObjectPath'}) {
        $hParams->{'ObjectPath'} = $Object->pathString;
    }
    else {
        $hParams->{'ObjectID'} = $Object->id;
    }

    my $Url;
    if ($Type eq 'admin') {
        if ($hOptions->{'UseSSL'}) {
            $Url = $Shop->get('WebServerScriptAdminUrlSSL');
        }
        else {
            $Url = $Shop->get('WebServerScriptAdminUrl');
        }
    }
    else { # 'sf', 'mobile', 'sf/mobile'
        if ($hOptions->{'UseSSL'}) {
            $Url = $Shop->get('WebServerScriptUrlSSL');
        }
        else {
            $Url = $Shop->get('WebServerScriptUrl');
        }
        my $isMobile = 0;
        if ($Type eq 'mobile') {
            $isMobile = 1;
        }
        elsif ($Type eq 'sf/mobile' && defined ($hOptions->{'IsMobileServlet'})) {
            $isMobile = $hOptions->{'IsMobileServlet'} ;
        }

        if( $isMobile ) {
            $Url = DE_EPAGES::Presentation::Servlet::Permission->mobileURL( $Url );
        }
    }

    # security token
    if ($hOptions->{'AddToken'} && defined($hOptions->{'Token'})) {
        $Url .= '/' . $hOptions->{'Token'};
    }

    # cookies
    my $SessionString;
    if ($hOptions->{'AddCookies'} && defined($hOptions->{'Cookies'})) {
        $SessionString = _BuildSessionString($hOptions->{'Cookies'});
    }
    else {
        $SessionString = _BuildSessionString(undef);
    }
    $SessionString .= BuildParameterString($hParams, $hOptions->{'NoUTF8'}) . '&';

    if ($Url !~ s/\/\?/$SessionString/) { #Store.sf/?ObjectPath=/Shops/Test
        $Url .= $SessionString; #short.sf
    }
    # remove ? or & at the end
    $Url =~ s/[\?&]$//;

    return $Url;
}

#========================================================================================
sub _BuildSessionString {
    my ($hCookies) = @_;

    my $SessionString = '/?';
    if (defined $hCookies) {
        #--- cookie values
        my $CookieString = join(';', map { UrlEncodeValue($_) . '=' . UrlEncodeValue($hCookies->{$_}//'') } keys %$hCookies);
        if (length($CookieString) > 0) {
            $SessionString = "/;$CookieString?";
        }
    }
    return $SessionString;
}

1;
