#========================================================================================
# package      DE_EPAGES::Shop::Hooks::ShopLanguage
# state        public
# description  copies the default shop name when a shop language is added
#========================================================================================
package DE_EPAGES::Shop::Hooks::ShopLanguage;

use strict;

use HTTP::Status qw( HTTP_NOT_FOUND HTTP_BAD_REQUEST );

#========================================================================================
# function     OnInsertShopLanguage
#----------------------------------------------------------------------------------------
# syntax       OnInsertShopLanguage( $hParams );
#----------------------------------------------------------------------------------------
# description  creates a new unity shop for the correspondenting locale and
#               copies the default shop name when a shop language is added
#               (Hook OBJ_InsertShopLanguage)
#----------------------------------------------------------------------------------------
# input        $hParams | hook params, keys:
#               <ul>
#                 <li>Object         - ShopLanguage         - object
#                 <li>ShopLanguageID - object id            - int
#               </ul>
#========================================================================================
sub OnInsertShopLanguage {
    my ($hParams) = @_;

    my $ShopLanguage = $hParams->{'Object'};
    my $Shop = $ShopLanguage->getSite();

    if ($Shop->get('IsUnityRegistered')) {
        my $NewLanguageID = $ShopLanguage->get('LanguageID');
        my $DefaultLanguageID = $Shop->get('LanguageID');

        my $ShopName = $Shop->get('Name', $DefaultLanguageID);

        # inform Unity about the new language (new unity shop is created)
        # get the locale id first wich will be added to the locales
        # the unity shop is created with this locale id and the name
        my $LocaleIDForNewShop = $Shop->localeForLanguageByShopLocaleIDAndLanguage($ShopLanguage);

        $Shop->sendUnityRequest('POST', '/shop/locale', undef, {'title'  => $ShopName,
                                                                'locale' => $LocaleIDForNewShop});

        $Shop->set({'Name' => $ShopName}, $NewLanguageID);

        # import uneditable attributes' translations
        $Shop->importUneditableAttributesUnityTranslations($NewLanguageID);
    }

    return;
}

#========================================================================================
# function     OnDeleteShopLanguage
#----------------------------------------------------------------------------------------
# syntax       OnDeleteShopLanguage( $hParams );
#----------------------------------------------------------------------------------------
# description  deletes the unity shop with the correspondenting locale
#               (Hook OBJ_InsertShopLanguage)
#----------------------------------------------------------------------------------------
# input        $hParams | hook params
#               <ul>
#                 <li>Object         - ShopLanguage         - object
#                 <li>ShopLanguageID - object id            - int
#               </ul>
#========================================================================================
sub OnDeleteShopLanguage {
    my ($hParams) = @_;

    my $ShopLanguage = $hParams->{'Object'};
    my $Shop = $ShopLanguage->getSite();

    if ($Shop->get('IsUnityRegistered')) {
        # inform Unity about the deleted language (unity shop with this language is deleted)
        my $LocaleIDForDeleteShop = $Shop->localeForLanguageByShopLocaleIDAndLanguage($ShopLanguage);
        $Shop->sendUnityRequest('DELETE', '/shop/locale/' . $LocaleIDForDeleteShop, undef, undef, [HTTP_NOT_FOUND,HTTP_BAD_REQUEST]);
    }

    return;
}

1;
